@@ -1,4 +1,5 @@
 import os
+import sys
 import time
 import socket
 import tempfile
@@ -16,49 +17,63 @@ except DistributionNotFound:
 
 hookimpl = pluggy.HookimplMarker("tox")
 
+HELP = ("Specify version constraints for packages which are then applied by "
+        "setting up a proxy PyPI server. If giving multiple constraints, you "
+        "can separate them with semicolons (;).")
+
+HELP_REQ = ("Specify version constraints for packages which are then applied by "
+            "setting up a proxy PyPI server. This should be set to a file in "
+            "pip requirements.txt format, i.e. one package per line, and can "
+            "be a URL.")
+
 
 @hookimpl
 def tox_addoption(parser):
-    parser.add_argument('--pypi-filter', dest='pypi_filter',
-                        help=('Like --force-dep, but will be applied regardless '
-                              'of whether the dependency is declared or not in '
-                              'the tox.ini file. This can be used to pin/constrain '
-                              'package versions for packages defined in e.g. '
-                              'extras_requires or even ones that are dependencies '
-                              'of specified dependencies. If giving multiple constraints, '
-                              'you can separate them with semicolons (;).'))
+
+    parser.add_argument('--pypi-filter', dest='pypi_filter', help=HELP)
     parser.add_argument('--pypi-filter-requirements', dest='pypi_filter_req',
-                        help=('Like --force-dep, but will be applied regardless '
-                              'of whether the dependency is declared or not in '
-                              'the tox.ini file. This can be used to pin/constrain '
-                              'package versions for packages defined in e.g. '
-                              'extras_requires or even ones that are dependencies '
-                              'of specified dependencies. Can be a local '
-                              'requirements file or a URL to one.'))
+                        help=HELP_REQ)
+
+    parser.add_testenv_attribute('pypi_filter', 'string', help=HELP)
+    parser.add_testenv_attribute('pypi_filter_requirements', 'string',
+                                 help=HELP_REQ)
 
 
 SERVER_PROCESS = None
 
 
 @hookimpl
-def tox_configure(config):
+def tox_get_python_executable(envconfig):
+
+    # This is a good place to set up the server an on environment by environment
+    # basis (since the pypi_filter configuration could in principle be different
+    # for each environment)
 
     global SERVER_PROCESS
 
-    if config.option.pypi_filter is None and config.option.pypi_filter_req is None:
+    # If running multiple environments, we need to shut down any previous server
+
+    if SERVER_PROCESS is not None:
+        SERVER_PROCESS.terminate()
+        SERVER_PROCESS = None
+
+    pypi_filter = envconfig.config.option.pypi_filter or envconfig.pypi_filter
+    pypi_filter_req = envconfig.config.option.pypi_filter_req or envconfig.pypi_filter_requirements
+
+    if pypi_filter is None and pypi_filter_req is None:
         return
 
-    if config.option.pypi_filter and config.option.pypi_filter_req:
+    if pypi_filter and pypi_filter_req:
         raise ValueError("Please specify only one of --pypi-filter or --pypi-filter-requirements")
 
-    if config.option.pypi_filter:
+    if pypi_filter:
         # Write out requirements to file
         reqfile = tempfile.mktemp()
         with open(reqfile, 'w') as f:
-            f.write(os.linesep.join(config.option.pypi_filter.split(';')))
+            f.write(os.linesep.join(pypi_filter.split(';')))
 
-    if config.option.pypi_filter_req:
-        url_info = urllib.parse.urlparse(config.option.pypi_filter_req)
+    if pypi_filter_req:
+        url_info = urllib.parse.urlparse(pypi_filter_req)
         if url_info.scheme:
             reqfile, _ = urllib.request.urlretrieve(url_info.geturl())
         else:
@@ -78,9 +93,7 @@ def tox_configure(config):
 
     # Run pypicky
     print('Starting tox-pypi-filter server with the following requirements:')
-    print('')
     print(contents)
-    print('')
 
     SERVER_PROCESS = subprocess.Popen([sys.executable, '-m', 'pypicky',
                                        reqfile, '--port', str(port), '--quiet'])
@@ -88,11 +101,13 @@ def tox_configure(config):
     # FIXME: properly check that the server has started up
     time.sleep(2)
 
-    config.indexserver['default'].url = f'http://localhost:{port}'
+    envconfig.config.indexserver['default'].url = f'http://localhost:{port}'
 
 
 @hookimpl
 def tox_cleanup(session):
+    global SERVER_PROCESS
     if SERVER_PROCESS is not None:
         print('Shutting down tox-pypi-filter server')
         SERVER_PROCESS.terminate()
+        SERVER_PROCESS = None
